# -*- coding: utf-8 -*-
# Generated by Django 1.11.10 on 2018-08-10 11:27
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sw_logger', '0003_auto_20170624_1913'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='log',
            options={'ordering': ('id',)},
        ),
        migrations.AddField(
            model_name='log',
            name='fk_object_id',
            field=models.IntegerField(db_index=True, null=True),
        ),
        migrations.AlterField(
            model_name='log',
            name='action',
            field=models.CharField(choices=[('', ''), ('created', 'created'), ('updated', 'updated'), ('deleted', 'deleted'), ('other', 'other')], default='', max_length=10),
        ),
    ]
